from .label import LabelSerializer
from .image_classification_annotation import ImageClassificationAnnotationSerializer

__all__ = [
    'LabelSerializer',
    'ImageClassificationAnnotationSerializer',
]